@@ -14,28 +14,10 @@ use reqwest::header::HeaderMap;
 pub mod drupal;
 pub mod text;
 
-/// Define one or more items to be validated in a web page response.
+/// Define one or more items to be validated in a web page response. For complete
+/// documentation, refer to [`ValidateBuilder`].
 ///
 /// This structure is passed to [`validate_and_load_static_assets`].
-///
-/// # Example
-/// ```rust
-/// use goose_eggs::Validate;
-///
-/// fn examples() {
-///     // Manually build a Validate strucuture that validates the page title and
-///     // some arbitrary texts in the response html.
-///     let _validate = Validate::new(
-///         None, Some("my page"), vec!["foo", r#"<a href="bar">"#], vec![], None
-///     );
-///
-///     // Use `title_texts()` helper to perform the same validation.
-///     let _validate = Validate::title_texts("my page", vec!["foo", r#"<a href="bar">"#]);
-///
-///     // Use `title_text()` helper to perform similar validation, validating only
-///     // one text on the page.
-///     let _validate = Validate::title_text("my page", r#"<a href="foo">"#);
-/// }
 #[derive(Clone, Debug)]
 pub struct Validate<'a> {
     /// Optionally validate the response status code.
@@ -45,48 +27,17 @@ pub struct Validate<'a> {
     /// Optionally validate arbitrary texts in the response html.
     texts: Vec<&'a str>,
     /// Optionally validate the response headers.
-    headers: Vec<&'a Header<'a>>,
+    headers: Vec<(&'a str, &'a str)>,
     /// Optionally validate whether or not the page redirects
     redirect: Option<bool>,
 }
 impl<'a> Validate<'a> {
-    /// Create a new Validate struct, specifying `status`, `title`, `texts`, `headers`,
-    /// and `redirect`.
-    ///
-    /// # Example
-    /// ```rust
-    /// use goose_eggs::{Header, Validate};
-    ///
-    /// let _validate = Validate::new(
-    ///     // Validate the response status code.
-    ///     Some(404),
-    ///     // Validate the response page title.
-    ///     Some("Page Not Found"),
-    ///     // Validate arbitrary text on the response page.
-    ///     vec!["Oops, something went wrong!"],
-    ///     // Validate that the response was sent via https.
-    ///     vec![&Header::name_value("scheme", "https")],
-    ///     // Validate that the page redirects.
-    ///     Some(true),
-    /// );
-    /// ```
-    pub fn new(
-        status: Option<u16>,
-        title: Option<&'a str>,
-        texts: Vec<&'a str>,
-        headers: Vec<&'a Header<'a>>,
-        redirect: Option<bool>,
-    ) -> Validate<'a> {
-        Validate {
-            status,
-            title,
-            texts,
-            headers,
-            redirect,
-        }
+    /// Convenience function to bring [`ValidateBuilder`] into scope.
+    pub fn builder() -> ValidateBuilder<'a> {
+        ValidateBuilder::new()
     }
 
-    /// Create a new Validate struct that validates nothing.
+    /// Create a [`Validate`] object that performs no validation.
     ///
     /// This is useful to load all static assets and return the body of the response.
     ///
@@ -99,251 +50,230 @@ impl<'a> Validate<'a> {
     /// let _validate = Validate::none();
     /// ```
     pub fn none() -> Validate<'a> {
-        Validate::new(None, None, vec![], vec![], None)
+        Validate::builder().build()
+    }
+}
+
+/// Used to build a [`Validate`] object, necessary to invoke the
+/// [`validate_and_load_static_assets`] function.
+///
+/// # Example
+/// ```rust
+/// use goose::prelude::*;
+/// use goose_eggs::{validate_and_load_static_assets, Validate};
+///
+/// task!(load_and_validate_page);
+///
+/// async fn load_and_validate_page(user: &mut GooseUser) -> GooseTaskResult {
+///     // Make a GET request.
+///     let mut goose = user.get("example/path").await?;
+///
+///     // Build a [`Validate`] object to confirm the response is valid.
+///     let validate = &Validate::builder()
+///         // Validate that the page has `Example` in the title.
+///         .title("Example")
+///         // Validate that the page has `foo` in the returned html body.
+///         .text("foo")
+///         // Validate that the page also has `<a href="bar">` in the returned
+///         // html body.
+///         .text(r#"<a href="bar">"#)
+///         .build();
+///
+///     // Perform the actual validation, using `?` to pass up the error if any
+///     // validation fails.
+///     validate_and_load_static_assets(
+///         user,
+///         goose,
+///         &validate,
+///     ).await?;
+///
+///     Ok(())
+/// }
+#[derive(Clone, Debug)]
+pub struct ValidateBuilder<'a> {
+    /// Optionally validate the response status code.
+    status: Option<u16>,
+    /// Optionally validate the response title.
+    title: Option<&'a str>,
+    /// Optionally validate arbitrary texts in the response html.
+    texts: Vec<&'a str>,
+    /// Optionally validate the response headers.
+    headers: Vec<(&'a str, &'a str)>,
+    /// Optionally validate whether or not the page redirects
+    redirect: Option<bool>,
+}
+impl<'a> ValidateBuilder<'a> {
+    // Internally used when building to set defaults.
+    fn new() -> Self {
+        Self {
+            status: None,
+            title: None,
+            texts: vec![],
+            headers: vec![],
+            redirect: None,
+        }
     }
 
-    /// Create a Validate object to validate the response status code.
+    /// Define the HTTP status expected to be returned when loading the page.
     ///
     /// This structure is passed to [`validate_and_load_static_assets`].
     ///
     /// # Example
     /// ```rust
-    /// use goose_eggs::{Header, Validate};
+    /// use goose_eggs::Validate;
     ///
-    /// let _validate = Validate::status(200);
+    /// let _validate = Validate::builder()
+    ///     .status(200)
+    ///     .build();
     /// ```
-    pub fn status(status: u16) -> Validate<'a> {
-        Validate::new(Some(status), None, vec![], vec![], None)
+    pub fn status(mut self, status: u16) -> Self {
+        self.status = Some(status);
+        self
     }
 
-    /// Create a Validate object to validate the response title.
+    /// Create a [`Validate`] object to validate that response title contains the specified
+    /// text.
     ///
     /// This structure is passed to [`validate_and_load_static_assets`].
     ///
     /// # Example
     /// ```rust
-    /// use goose_eggs::{Header, Validate};
+    /// use goose_eggs::Validate;
     ///
-    /// let _validate = Validate::title("Home page");
+    /// let _validate = Validate::builder()
+    ///     .title("Home page")
+    ///     .build();
     /// ```
-    pub fn title(title: &'a str) -> Validate<'a> {
-        Validate::new(None, Some(title), vec![], vec![], None)
+    pub fn title(mut self, title: impl Into<&'a str>) -> Self {
+        self.title = Some(title.into());
+        self
     }
 
-    /// Create a Validate object to validate specific text is on the response page.
+    /// Create a [`Validate`] object to validate that the response page contains the specified
+    /// text.
     ///
     /// This structure is passed to [`validate_and_load_static_assets`].
     ///
     /// # Example
     /// ```rust
-    /// use goose_eggs::{Header, Validate};
+    /// use goose_eggs::Validate;
     ///
-    /// let _validate = Validate::text("This text should be on the page.");
+    /// let _validate = Validate::builder()
+    ///     .text("example")
+    ///     .build();
     /// ```
-    pub fn text(text: &'a str) -> Validate<'a> {
-        Validate::new(None, None, vec![text], vec![], None)
-    }
-
-    /// Create a Validate object to validate the response has the correct title and also
-    /// contains specific text.
     ///
-    /// This structure is passed to [`validate_and_load_static_assets`].
+    /// It's possible to call this function multiple times to validate that multiple texts
+    /// appear on the page. Alternatively you can call [`ValidateBuilder::texts`].
     ///
-    /// # Example
+    /// # Multiple Example
     /// ```rust
-    /// use goose_eggs::{Header, Validate};
+    /// use goose_eggs::Validate;
     ///
-    /// let _validate = Validate::title_text("Example", "This text should be on the page.");
+    /// let _validate = Validate::builder()
+    ///     .text("example")
+    ///     .text("another")
+    ///     .build();
     /// ```
-    pub fn title_text(title: &'a str, text: &'a str) -> Validate<'a> {
-        Validate::new(None, Some(title), vec![text], vec![], None)
+    pub fn text(mut self, text: &'a str) -> Self {
+        self.texts.push(text);
+        self
     }
 
-    /// Create a Validate object to validate that the response page contains multiple specific
+    /// Create a [`Validate`] object to validate that the response page contains the specified
     /// texts.
     ///
     /// This structure is passed to [`validate_and_load_static_assets`].
     ///
     /// # Example
     /// ```rust
-    /// use goose_eggs::{Header, Validate};
-    ///
-    /// let _validate = Validate::texts(vec!["This text should be on the page.", "And also this", r#"<span>and this</a>"#]);
-    /// ```
-    pub fn texts(texts: Vec<&'a str>) -> Validate<'a> {
-        Validate::new(None, None, texts, vec![], None)
-    }
-
-    /// Create a Validate object to validate the response title and that the page contains
-    /// multiple specific texts.
-    ///
-    /// This structure is passed to [`validate_and_load_static_assets`].
-    ///
-    /// # Example
-    /// ```rust
-    /// use goose_eggs::{Header, Validate};
-    ///
-    /// let _validate = Validate::title_texts("Example", vec!["This text should be on the page.", "And also this", r#"<span>and this</a>"#]);
-    /// ```
-    pub fn title_texts(title: &'a str, texts: Vec<&'a str>) -> Validate<'a> {
-        Validate::new(None, Some(title), texts, vec![], None)
-    }
-
-    /// Create a Validate object to validate the response included a specific header.
-    ///
-    /// This structure is passed to [`validate_and_load_static_assets`].
-    ///
-    /// # Example
-    /// ```rust
-    /// use goose_eggs::{Header, Validate};
-    ///
-    /// let _validate = Validate::header(&Header::name("x-cache"));
-    /// ```
-    pub fn header(header: &'a Header<'a>) -> Validate<'a> {
-        Validate::new(None, None, vec![], vec![header], None)
-    }
-
-    /// Create a Validate object to validate that the response included multiple specific headers.
-    ///
-    /// This structure is passed to [`validate_and_load_static_assets`].
-    ///
-    /// # Example
-    /// ```rust
-    /// use goose_eggs::{Header, Validate};
-    ///
-    /// let _validate = Validate::headers(vec![&Header::name("x-cache"), &Header::name("x-generator")]);
-    /// ```
-    pub fn headers(headers: Vec<&'a Header<'a>>) -> Validate<'a> {
-        Validate::new(None, None, vec![], headers, None)
-    }
-
-    /// Create a Validate object to validate whether or not the response redirected.
-    ///
-    /// This structure is passed to [`validate_and_load_static_assets`].
-    ///
-    /// # Example
-    /// ```rust
     /// use goose_eggs::Validate;
     ///
-    /// // Verify the response redirected.
-    /// let _validate = Validate::redirect(true);
-    ///
-    /// // Verify the response did not redirect.
-    /// let _validate = Validate::redirect(false);
+    /// let _validate = Validate::builder()
+    ///     .texts(vec!["example", "another"])
+    ///     .build();
     /// ```
-    pub fn redirect(redirect: bool) -> Validate<'a> {
-        Validate::new(None, None, vec![], vec![], Some(redirect))
-    }
-
-    /// Update a Validate object, changing the expected status returned when loading the page.
     ///
-    /// This structure is passed to [`validate_and_load_static_assets`].
-    ///
-    /// # Example
-    /// ```rust
-    /// use goose_eggs::{Header, Validate};
-    ///
-    /// let _validate = Validate::title("Home page")
-    ///                     .update_status(200);
-    /// ```
-    pub fn update_status(mut self, status: u16) -> Self {
-        self.status = Some(status);
-        self
-    }
-
-    /// Update a Validate object, changing the title expected on the page.
-    ///
-    /// This structure is passed to [`validate_and_load_static_assets`].
-    ///
-    /// # Example
-    /// ```rust
-    /// use goose_eggs::{Header, Validate};
-    ///
-    /// let _validate = Validate::status(200)
-    ///                     .update_title("Home Page");
-    /// ```
-    pub fn update_title(mut self, title: &'a str) -> Self {
-        self.title = Some(title);
-        self
-    }
-
-    /// Update a Validate object, changing the text validated on the page.
-    ///
-    /// Use [`Validate::update_texts`] if you want to validate multiple texts
-    /// on the page.
-    ///
-    /// This structure is passed to [`validate_and_load_static_assets`].
-    ///
-    /// # Example
-    /// ```rust
-    /// use goose_eggs::{Header, Validate};
-    ///
-    /// let _validate = Validate::title("Home Page")
-    ///                     .update_text("Welcome to my Home Page");
-    /// ```
-    pub fn update_text(mut self, text: &'a str) -> Self {
-        self.texts = vec![text];
-        self
-    }
-
-    /// Update a Validate object, changing the texts validated on the page.
-    ///
-    /// Use [`Validate::update_text`] if you want to validate only one text
-    /// on the page.
-    ///
-    /// This structure is passed to [`validate_and_load_static_assets`].
-    ///
-    /// # Example
-    /// ```rust
-    /// use goose_eggs::{Header, Validate};
-    ///
-    /// let _validate = Validate::title("Home Page")
-    ///                     .update_texts(vec!["Welcome", "Log in"]);
-    /// ```
-    pub fn update_texts(mut self, texts: Vec<&'a str>) -> Self {
+    /// Alternatively you can call [`ValidateBuilder::text`].
+    pub fn texts(mut self, texts: Vec<&'a str>) -> Self {
         self.texts = texts;
         self
     }
 
-    /// Update a Validate object, changing the header expected on the page.
+    /// Create a [`Validate`] object to validate that the response includes the specified
+    /// header.
     ///
-    /// Use [`Validate::update_headers`] if you want to validate multiple headers
-    /// on the page.
+    /// To validate that a header contains a specific value (instead of just validating
+    /// that it exists), use [`ValidateBuilder::header_value`].
     ///
     /// This structure is passed to [`validate_and_load_static_assets`].
     ///
     /// # Example
     /// ```rust
-    /// use goose_eggs::{Header, Validate};
+    /// use goose_eggs::Validate;
     ///
-    /// let _validate = Validate::title("Home Page")
-    ///                     .update_header(&Header::name("x-cache"));
+    /// let _validate = Validate::builder()
+    ///     .header("x-cache")
+    ///     .build();
     /// ```
-    pub fn update_header(mut self, header: &'a Header<'a>) -> Self {
-        self.headers = vec![header];
+    ///
+    /// It's possible to call this function multiple times to validate that multiple
+    /// headers are set.
+    ///
+    /// # Multiple Example
+    /// ```rust
+    /// use goose_eggs::Validate;
+    ///
+    /// let _validate = Validate::builder()
+    ///     .header("x-cache")
+    ///     .header("x-generator")
+    ///     .build();
+    /// ```
+    pub fn header(mut self, header: impl Into<&'a str>) -> Self {
+        self.headers.push((header.into(), ""));
         self
     }
 
-    /// Update a Validate object, changing the headers expected on the page.
+    /// Create a [`Validate`] object to validate that the response includes the specified
+    /// header which contains the specified value.
     ///
-    /// Use [`Validate::update_header`] if you want to validate only one header
-    /// on the page.
+    /// To validate that a header simply exists without confirming that it contains a
+    /// specific value, use [`ValidateBuilder::header`].
     ///
     /// This structure is passed to [`validate_and_load_static_assets`].
     ///
     /// # Example
     /// ```rust
-    /// use goose_eggs::{Header, Validate};
+    /// use goose_eggs::Validate;
     ///
-    /// let _validate = Validate::title("Home Page")
-    ///                     .update_headers(vec![&Header::name("x-cache"), &Header::name("x-generator")]);
+    /// let _validate = Validate::builder()
+    ///     .header_value("x-generator", "Drupal 7")
+    ///     .build();
     /// ```
-    pub fn update_headers(mut self, headers: Vec<&'a Header<'a>>) -> Self {
-        self.headers = headers;
+    ///
+    /// It's possible to call this function multiple times, and/or together with
+    /// [`ValidateBuilder::header`] to validate that multiple headers are set and their
+    /// values.
+    ///
+    /// # Multiple Example
+    /// ```rust
+    /// use goose_eggs::Validate;
+    ///
+    /// let _validate = Validate::builder()
+    ///     // Validate that the "x-cache" header is set.
+    ///     .header("x-cache")
+    ///     // Validate that the "x-generator" header is set and contains "Drupal 7".
+    ///     .header_value("x-generator", "Drupal-7")
+    ///     // Validate that the "x-drupal-cache" header is set and contains "HIT".
+    ///     .header_value("x-drupal-cache", "HIT")
+    ///     .build();
+    /// ```
+    pub fn header_value(mut self, header: impl Into<&'a str>, value: impl Into<&'a str>) -> Self {
+        self.headers.push((header.into(), value.into()));
         self
     }
 
-    /// Update a Validate object, changing whether or not the page should redirect.
+    /// Create a [`Validate`] object to validate whether or not the response page redirected.
     ///
     /// This structure is passed to [`validate_and_load_static_assets`].
     ///
@@ -352,71 +282,43 @@ impl<'a> Validate<'a> {
     /// use goose_eggs::Validate;
     ///
     /// // Verify the response redirected.
-    /// let _validate = Validate::title("Home Page")
-    ///                     .update_redirect(false);
+    /// let _validate = Validate::builder().redirect(true).build();
+    ///
+    /// // Verify the response did not redirect.
+    /// let _validate = Validate::builder().redirect(false).build();
     /// ```
-    pub fn update_redirect(mut self, redirect: bool) -> Self {
-        self.redirect = Some(redirect);
+    pub fn redirect(mut self, redirect: impl Into<bool>) -> Self {
+        self.redirect = Some(redirect.into());
         self
     }
-}
 
-/// Used to validate that headers are included in the server response.
-///
-/// # Example
-/// ```rust
-/// use goose_eggs::Header;
-///
-/// fn example() {
-///     // Validate that the "x-varnish" header is set.
-///     let _header = Header::name("x-varnish");
-/// }
-
-#[derive(Clone, Debug)]
-pub struct Header<'a> {
-    /// The name of the header to validate, required.
-    name: &'a str,
-    /// The value of the header to validate, optional.
-    value: Option<&'a str>,
-}
-impl<'a> Header<'a> {
-    /// Create a new Header validation struct by specifying all fields.
+    /// Build the [`Validate`] object which is then passed to the
+    /// [`validate_and_load_static_assets`] function.
     ///
     /// # Example
     /// ```rust
-    /// use goose_eggs::Header;
+    /// use goose_eggs::Validate;
     ///
-    /// let _header = Header::new("foo", Some("bar"));
+    /// // Use the default search form to search for `example keys`.
+    /// let _validate = Validate::builder()
+    ///     .text("example text")
+    ///     .build();
     /// ```
-    pub fn new(name: &'a str, value: Option<&'a str>) -> Header<'a> {
-        Header { name, value }
-    }
-
-    /// Create a Header object to validate that a named header is set.
-    ///
-    /// # Example
-    /// ```rust
-    /// use goose_eggs::Header;
-    ///
-    /// // Create a Header object to validate that the "foo" header is set in the Response.
-    /// let _header = Header::name("foo");
-    /// ```
-    pub fn name(name: &'a str) -> Header<'a> {
-        Header::new(name, None)
-    }
-
-    /// Create a Header object to validate that a named header contains a specific value.
-    ///
-    /// # Example
-    /// ```rust
-    /// use goose_eggs::Header;
-    ///
-    /// // Create a Header object to validate that the "foo" header is set and contains "bar"
-    /// // in the Response.
-    /// let _header = Header::name_value("foo", "bar");
-    /// ```
-    pub fn name_value(name: &'a str, value: &'a str) -> Header<'a> {
-        Header::new(name, Some(value))
+    pub fn build(self) -> Validate<'a> {
+        let Self {
+            status,
+            title,
+            texts,
+            headers,
+            redirect,
+        } = self;
+        Validate {
+            status,
+            title,
+            texts,
+            headers,
+            redirect,
+        }
     }
 }
 
@@ -654,7 +556,7 @@ pub fn valid_text(html: &str, text: &str) -> bool {
 /// # Example
 /// ```rust
 /// use goose::prelude::*;
-/// use goose_eggs::{header_is_set, Header};
+/// use goose_eggs::header_is_set;
 ///
 /// task!(validate_header).set_on_start();
 ///
@@ -665,7 +567,7 @@ pub fn valid_text(html: &str, text: &str) -> bool {
 ///         Ok(response) => {
 ///             // Copy the headers so we have them for logging if there are errors.
 ///             let headers = &response.headers().clone();
-///             if !header_is_set(headers, &Header::name("server")) {
+///             if !header_is_set(headers, "server") {
 ///                 return user.set_failure(
 ///                     &format!("{}: header not found: {}", goose.request.raw.url, "server"),
 ///                     &mut goose.request,
@@ -687,21 +589,24 @@ pub fn valid_text(html: &str, text: &str) -> bool {
 ///     Ok(())
 /// }
 /// ```
-pub fn header_is_set(headers: &HeaderMap, header: &Header) -> bool {
-    headers.contains_key(header.name)
+pub fn header_is_set(headers: &HeaderMap, header: &str) -> bool {
+    headers.contains_key(header)
 }
 
 /// Returns a [`bool`] indicating whether or not a header contains an expected value.
 ///
 /// Returns [`true`] if the expected value was found, otherwise returns [`false`].
 ///
+/// Expects a [`&str`] [`tuple`] with a length of 2 where the first defines the header
+/// name and the second defines the header value, ie `("name", "value")`.
+///
 /// While you can invoke this function directly, it's generally preferred to invoke
 /// [`validate_and_load_static_assets`] which in turn invokes this function.
 ///
 /// # Example
 /// ```rust
 /// use goose::prelude::*;
-/// use goose_eggs::{valid_header_value, Header};
+/// use goose_eggs::valid_header_value;
 ///
 /// task!(validate_header_value).set_on_start();
 ///
@@ -712,7 +617,7 @@ pub fn header_is_set(headers: &HeaderMap, header: &Header) -> bool {
 ///         Ok(response) => {
 ///             // Copy the headers so we have them for logging if there are errors.
 ///             let headers = &response.headers().clone();
-///             if !valid_header_value(headers, &Header::name_value("server", "nginx")) {
+///             if !valid_header_value(headers, ("server", "nginx")) {
 ///                 return user.set_failure(
 ///                     &format!("{}: server header value not correct: {}", goose.request.raw.url, "nginx"),
 ///                     &mut goose.request,
@@ -734,30 +639,36 @@ pub fn header_is_set(headers: &HeaderMap, header: &Header) -> bool {
 ///     Ok(())
 /// }
 /// ```
-pub fn valid_header_value(headers: &HeaderMap, header: &Header) -> bool {
-    if header_is_set(headers, header) {
-        if let Some(value_to_validate) = header.value {
-            let header_value = match headers.get(header.name) {
+pub fn valid_header_value<'a>(headers: &HeaderMap, header: (&'a str, &'a str)) -> bool {
+    // A header name is required, exit early if it's empty.
+    if header.0.is_empty() {
+        info!("no header specified");
+        return false;
+    }
+
+    if header_is_set(headers, header.0) {
+        if header.1.is_empty() {
+            false
+        } else {
+            let header_value = match headers.get(header.0) {
                 // Extract the value of the header and try to convert to a &str.
                 Some(v) => v.to_str().unwrap_or(""),
                 None => "",
             };
             // Check if the desired value is in the header.
-            if header_value.contains(value_to_validate) {
+            if header_value.contains(header.1) {
                 true
             } else {
                 // Provide some extra debug.
                 info!(
                     r#"header does not contain expected value: "{}: {}""#,
-                    header.name, header_value
+                    header.0, header.1
                 );
                 false
             }
-        } else {
-            false
         }
     } else {
-        info!("header ({}) not set", header.name);
+        info!("header ({}) not set", header.0);
         false
     }
 }
@@ -840,9 +751,9 @@ pub async fn load_static_elements(user: &mut GooseUser, html: &str) {
 ///
 /// What is validated is defined with the [`Validate`] structure.
 ///
-/// If the page doesn't load, an empty String will be returned. If the page does load
+/// If the page doesn't load, an empty [`String`] will be returned. If the page does load
 /// but validation fails, an Error is returned. If the page loads and there are no
-/// errors the body is returned as a String.
+/// errors the body is returned as a [`String`].
 ///
 /// # Example
 /// ```rust
@@ -857,7 +768,10 @@ pub async fn load_static_elements(user: &mut GooseUser, html: &str) {
 ///         user,
 ///         goose,
 ///         // Validate title and other arbitrary text on the response html.
-///         &Validate::title_texts("my page", vec!["foo", r#"<a href="bar">"#]),
+///         &Validate::builder()
+///             .title("my page")
+///             .texts(vec!["foo", r#"<a href="bar">"#])
+///             .build(),
 ///     ).await?;
 ///
 ///     Ok(())
@@ -891,11 +805,11 @@ pub async fn validate_and_load_static_assets<'a>(
             }
 
             // Validate status code if defined.
-            let response_status = response.status();
             if let Some(status) = validate.status {
-                if response_status != status {
+                if response.status() != status {
                     // Get as much as we can from the response for useful debug logging.
                     let headers = &response.headers().clone();
+                    let response_status = response.status();
                     let html = response.text().await.unwrap_or_else(|_| "".to_string());
                     user.set_failure(
                         &format!(
@@ -915,7 +829,7 @@ pub async fn validate_and_load_static_assets<'a>(
             // Validate headers if defined.
             let headers = &response.headers().clone();
             for header in &validate.headers {
-                if !header_is_set(headers, header) {
+                if !header_is_set(headers, header.0) {
                     // Get as much as we can from the response for useful debug logging.
                     let html = response.text().await.unwrap_or_else(|_| "".to_string());
                     user.set_failure(
@@ -931,23 +845,21 @@ pub async fn validate_and_load_static_assets<'a>(
                     // errors when a page fails to load.
                     return Ok(html);
                 }
-                if let Some(h) = header.value {
-                    if !valid_header_value(headers, header) {
-                        // Get as much as we can from the response for useful debug logging.
-                        let html = response.text().await.unwrap_or_else(|_| "".to_string());
-                        user.set_failure(
-                            &format!(
-                                "{}: header does not contain expected value: {:?}",
-                                goose.request.raw.url, h
-                            ),
-                            &mut goose.request,
-                            Some(headers),
-                            Some(&html),
-                        )?;
-                        // Exit as soon as validation fails, to avoid cascades of
-                        // errors when a page fails to load.
-                        return Ok(html);
-                    }
+                if !header.1.is_empty() && !valid_header_value(headers, *header) {
+                    // Get as much as we can from the response for useful debug logging.
+                    let html = response.text().await.unwrap_or_else(|_| "".to_string());
+                    user.set_failure(
+                        &format!(
+                            "{}: header does not contain expected value: {:?}",
+                            goose.request.raw.url, header.1
+                        ),
+                        &mut goose.request,
+                        Some(headers),
+                        Some(&html),
+                    )?;
+                    // Exit as soon as validation fails, to avoid cascades of
+                    // errors when a page fails to load.
+                    return Ok(html);
                 }
             }
 
